<?php
        include_once("globals.inc");

        include_once("sql.inc");

        // this function creates the images table and sequence if necessary
        // (i.e. that they will be created if they don't exist)
        function prepareImagesTable($db) {
                global $strORDImageType, $strImagesTable, $strImagesColumn;
                prepareTable($db, $strORDImageType, $strImagesTable, $strImagesColumn);
        }

        function prepareThesaurus($db) {
                global $messages;

                $messages[] = "Um die Funktionalitaeten dieser PHP-Seite nutzen zu koennen, muessen Sie die Voraussetzungen fuer das Beispiel \"Agatha Christie\" des Uebungsblattes zu Oracle Text geschaffen haben. Diese PHP-Seite ueberprueft nicht, ob dem so ist.";
        }

        function preparePhotosTable($db) {
                global $messages;

                $messages[] = "Um die Funktionalitaeten dieser PHP-Seite nutzen zu koennen, muessen Sie die selben Voraussetzungen wie fuer den letzten Abschnitt des Uebungsblattes zu ORDImage geschaffen haben. Sie muessen also insbesondere eine Tabelle \"photos\", die mit Bildern gefuellt ist, in Ihrer Datenbank vorliegen haben. Diese PHP-Seite ueberprueft nicht, ob dem so ist.";
        }

        // this function creates the sounds table and sequence if necessary
        // (i.e. that they will be created if they don't exist)
        function prepareSoundsTable($db) {
                global $strORDAudioType, $strSoundsTable, $strSoundsColumn;
                prepareTable($db, $strORDAudioType, $strSoundsTable, $strSoundsColumn);
        }

        // this function creates the videos table and sequence if necessary
        // (i.e. that they will be created if they don't exist)
        function prepareVideosTable($db) {
                global $strORDVideoType, $strVideosTable, $strVideosColumn;
                prepareTable($db, $strORDVideoType, $strVideosTable, $strVideosColumn);
        }

        // this function is used by specialized functions like prepareImagesTable() to
        // create the necessary database structure
        function prepareTable($db, $type, $table, $column) {
                // we use the global messages array to inform the user about
                // things he should know
                global $messages;

                // construct a query to check if the images table is already there
                // (we need to specify the table name upper case, otherwise we wouldn't
                // match the actual table)
                $table = strtoupper($table);
                $sql = "SELECT COUNT(*) as count FROM user_tables WHERE table_name = '$table'";
                $stmt = OCIParse($db, $sql);

                OCIExecute($stmt);

                // assign a default value to the count variable
                $count = 0;

                if (OCIFetch($stmt)) {
                        $count = OCIResult($stmt, 1);
                } else {
                        $oerr = OCIError($stmt);
                        $messages[] = 'database error: ' . $oerr["message"];
                }

                // if the images table is not there, we will create it
                // (if it does not exist, we assume that the corresponding
                // sequence isn't there too)
                if ($count == 0) {

                        // construct the create table statement
                        $sql =
                                "CREATE TABLE $table (
                                        id        int,
                                        $column        ORDSYS.$type,
                                        primary key (id)
                                )";
                        $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Created table '$table'.";
                        }

                        // construct the create sequence statement
                        $sql = "CREATE SEQUENCE {$table}_seq";
                        $stmt = OCIParse($db, $sql);

                        if (OCIExecute($stmt)) {
                                $messages[] = "Created sequence '{$table}_seq' successfully.";
                        } else {
                                $oerr = OCIError($stmt);
                                $messages[] = "Error when creating sequence '{$table}_seq': " . $oerr['message'];
                        }
                }
        }

        // this function creates the necessary database structure for Oracle Text testing
        function prepareTextsTable($db) {
                // we use the global messages array to inform the user about
                // things he should know
                global $messages, $strTextsTable, $strTextsTable_title, $strTextsTable_pk;
                global $sqlAgathaChristie, $sqlAgathaChristieCreatePreferences, $sqlAgathaChristieDropPreferences;

                // construct a query to check if the texts table is already there
                // (we need to specify the table name upper case, otherwise we wouldn't
                // match the actual table)
                $upperCase = strtoupper($strTextsTable);
                $sql = "SELECT COUNT(*) AS count FROM user_tables WHERE table_name = '$upperCase'";
                $stmt = OCIParse($db, $sql);

                OCIExecute($stmt);

                // assign a default value to the count variable
                $count = 0;

                if (OCIFetch($stmt)) {
                        $count = OCIResult($stmt, 1);
                } else {
                        $oerr = OCIError($stmt);
                        $messages[] = 'database error: ' . $oerr["message"];
                }

                // if the texts table is not there, we will create it
                if ($count == 0) {
                        $sql = "create table $strTextsTable (
                                        id number,
                                        title varchar2(700)
                                )";
                        $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Created table '$strTextsTable'.";
                        }

                        // read the necessary sql statements from the specified file
                        $sql = file_get_contents($sqlAgathaChristie);
                        // remove carriage return and line feed (CR-LF)
                        $sql = strtr ($sql,chr(13).chr(10)," ");
                        $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Inserted data.";
                        }

                        $sql = "alter table christie
                                          add constraint $strTextsTable_pk primary key ( id )
                                          using index";
                          $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Altered table '$strTextsTable'.";
                        }

                        // $sql = "create unique index christie_title on christie ( title )";
                        // $sql = "drop index christie_title";

                        // read the necessary sql statements from the specified file
                        $sql = file_get_contents($sqlAgathaChristieCreatePreferences);
                        // remove carriage return and line feed (CR-LF)
                        $sql = strtr ($sql,chr(13).chr(10)," ");
                        $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Created preferences.";
                        }

                        $sql = "create index $strTextsTable_title on $strTextsTable ( title )
                                          indextype is ctxsys.context
                                          parameters ( 'lexer my_basic_lexer section group my_basic_section_group wordlist my_basic_wordlist storage mystore' )";
                          $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Created index '{$strTextsTable}_title'.";
                        }

                        // read the necessary sql statements from the specified file
                        $sql = file_get_contents($sqlAgathaChristieDropPreferences);
                        // remove carriage return and line feed (CR-LF)
                        $sql = strtr ($sql,chr(13).chr(10)," ");
                        $stmt = OCIParse($db, $sql);
                        if (OCIExecute($stmt)) {
                                $messages[] = "Dropped preferences.";
                        }
                }
        }

        // this function uploads the given image (specified by the files name and path)
        // to the database
        function uploadImage(&$db, &$file) {
                global $strORDImageType, $strImagesTable, $strImagesColumn;

                $rowid = uploadData($db, $file, $strORDImageType, $strImagesTable, $strImagesColumn);
                extractImageProperties($db, $rowid, $strORDImageType, $strImagesTable, $strImagesColumn);
        }

        // this function uploads the given video (specified by the files name and path)
        // to the database
        function uploadVideo(&$db, &$file) {
                global $strORDVideoType, $strVideosTable, $strVideosColumn;

                $rowid = uploadData($db, $file, $strORDVideoType, $strVideosTable, $strVideosColumn);
                extractSoundOrVideoProperties($db, $rowid, $strORDVideoType, $strVideosTable, $strVideosColumn);
        }

        // this function uploads the given sound (specified by the files name and path)
        // to the database
        function uploadSound(&$db, &$file) {
                global $strORDAudioType, $strSoundsTable, $strSoundsColumn;

                $rowid = uploadData($db, $file, $strORDAudioType, $strSoundsTable, $strSoundsColumn);
                extractSoundOrVideoProperties($db, $rowid, $strORDAudioType, $strSoundsTable, $strSoundsColumn);
        }

        // this function is used by specialized functions like uploadImage() to upload
        // media to the database; it returns the rowid of the newly inserted record which
        // is used to reference the record later on
        function uploadData($db, $file, $type, $table, $column) {

                // insert the new record into the media's table and load the
                // corresponding blob with the media's data
                // (we use oracle's pseudo column rowid which identifies a row
                // within a table (but not within a database) to refer to the
                // right record later on)
                $sql ="DECLARE
                                obj ORDSYS.$type;
                                iblob BLOB;
                        BEGIN
                                INSERT INTO $table (id, $column) VALUES
                                ({$table}_seq.nextval, ordsys.$type.init())
                                RETURNING ROWID INTO :rid;

                                SELECT $column INTO obj FROM $table
                                WHERE ROWID = :rid FOR UPDATE;

                                iblob := obj.source.localData;
                                :extblob := iblob;

                                UPDATE $table SET $column = obj WHERE ROWID = :rid;
                        END;";

                // the function OCINewDescriptor allocates storage to hold descriptors or
                // lob locators.
                // see http://www.php.net/manual/en/function.ocinewdescriptor.php
                $blob = OCINewDescriptor ($db, OCI_D_LOB);
                $rowid = OCINewDescriptor($db, OCI_D_ROWID);

                $sql = strtr ($sql,chr(13).chr(10)," ");
                $stmt = OCIParse($db, $sql);

                // the function OCIBindByName binds a PHP variable to a oracle placeholder
                // (whether the variable will be used for input or output will be determined
                // run-time, and the necessary storage space will be allocated)
                // see http://www.php.net/manual/en/function.ocibindbyname.php
                OCIBindByName($stmt, ':extblob', $blob, -1, OCI_B_BLOB);
                OCIBindByName($stmt, ':rid', $rowid, -1, OCI_B_ROWID);

                OCIExecute ($stmt, OCI_DEFAULT);

                // read the files data and load it into the blob
                $blob->savefile($file);

                OCIFreeStatement ($stmt);
                $blob->free();

                return $rowid;
        }

        // extract properties of the inserted media so that we can use db-functions
        // like getHeight(), getFileFormat() and so on to gather information about the
        // media's metadata
        function extractImageProperties($db, $rowid, $type, $table, $column) {
                $sql = "DECLARE
                                obj ORDSYS.$type;
                        BEGIN
                                SELECT $column INTO obj FROM $table WHERE ROWID = :rid FOR UPDATE;
                                obj.setProperties;
                                UPDATE $table SET $column = obj WHERE ROWID = :rid;
                        END;";
                $sql = strtr ($sql,chr(13).chr(10)," ");
                $stmt = OCIParse($db, $sql);
                OCIBindByName($stmt, ':rid', $rowid, -1, OCI_B_ROWID);
                OCIExecute ($stmt, OCI_DEFAULT);
        }

        // extract properties of the inserted media so that we can use functions like
        // getHeight(), getFileFormat() and so on to gather information about the
        // media's metadata
        function extractSoundOrVideoProperties($db, $rowid, $type, $table, $column) {
                $sql = "DECLARE
                                obj ORDSYS.$type;
                                ctx RAW(4000) := NULL;
                        BEGIN
                                SELECT $column INTO obj FROM $table WHERE ROWID = :rid FOR UPDATE;
                                obj.setProperties(ctx);
                                UPDATE $table SET $column = obj WHERE ROWID = :rid;
                        END;";
                $sql = strtr ($sql,chr(13).chr(10)," ");
                $stmt = OCIParse($db, $sql);
                OCIBindByName($stmt, ':rid', $rowid, -1, OCI_B_ROWID);
                OCIExecute ($stmt, OCI_DEFAULT);
        }

        // loads an image from the database
        function retrieveImage($db, $id, $table, &$data, &$mimetype) {
                global $strORDImageType, $strImagesColumn;
                retrieveData($db, $id, $data, $mimetype, $strORDImageType, $table, $strImagesColumn);
        }

        // loads a sound from the database
        function retrieveSound($db, $id, &$data, &$mimetype) {
                global $strORDAudioType, $strSoundsTable, $strSoundsColumn;
                retrieveData($db, $id, $data, $mimetype, $strORDAudioType, $strSoundsTable, $strSoundsColumn);
        }

        // loads a video from the database
        function retrieveVideo($db, $id, &$data, &$mimetype) {
                global $strORDVideoType, $strVideosTable, $strVideosColumn;
                retrieveData($db, $id, $data, $mimetype, $strORDVideoType, $strVideosTable, $strVideosColumn);
        }

        // this function is used by specialized functions like retrieveImage() to load
        // a media from the database
        function retrieveData($db, $id, &$data, &$mimetype, $type, $table, $column) {
                // the function OCINewDescriptor allocates storage to hold descriptors or
                // lob locators,
                // see http://www.php.net/manual/en/function.ocinewdescriptor.php
                $blob = OCINewDescriptor ($db, OCI_D_LOB);

                // construct the sql query with which we will get the media's data
                $sql = "DECLARE
                                obj ORDSYS.$type;
                        BEGIN
                                SELECT $column INTO obj FROM $table WHERE id = :id;
                                :mimeType := obj.getMimeType;
                                :extblob := obj.getContent;
                        END;";
                $sql = strtr ($sql,chr(13).chr(10)," ");

                $stmt = OCIParse($db, $sql);

                // the function OCIBindByName binds a PHP variable to a oracle placeholder
                // (wheter the variable will be used for input or output will be determined
                // run-time, and the necessary storage space will be allocated)
                // see http://www.php.net/manual/en/function.ocibindbyname.php
                OCIBindByName ($stmt, ':extBlob', $blob, -1, OCI_B_BLOB);
                OCIBindByName ($stmt, ':mimeType', $mimetype, 20);
                OCIBindByName ($stmt, ':id', $id);

                OCIExecute ($stmt, OCI_DEFAULT);

                // load the binary data
                $data = $blob->load();
        }

        // this function simply executes a query and generates a header array
        // which contains the column names and an array which contains the actual
        // data
        function executeQuery($db, $sql, &$header, &$arr) {
                $stmt = OCIParse($db, $sql);
                OCIExecute($stmt);

                $ncols = OCINumCols($stmt);
                for ($i = 1; $i <= $ncols; $i++) {
                        $header[]  = OCIColumnName($stmt, $i);
                }

                // construct an array which will be displayed later
                /* Veraltet while (OCIFetchInto($stmt, $row, OCI_ASSOC)) {
                  if ($row) {
                                $arr[] = $row;
                        } else {
                                break;
                        }
                } */

                // Neuer:
                while ($row = oci_fetch_assoc($stmt)) {
                                $arr[] = $row;
                }
        }

        function setDecimalPoint($db) {
                $sql = "ALTER SESSION SET NLS_NUMERIC_CHARACTERS = \".,\"";
                $stmt = OCIParse($db, $sql);
                if (OCIExecute($stmt)) {
                        return true;
                } else {
                        return false;
                }
        }
?>